@@ -8,22 +8,22 @@ from __future__ import division
 import itertools
 import numbers
 import numpy as np
-from warnings import warn
 from abc import ABCMeta, abstractmethod
+from warnings import warn
 
+from .base import BaseEnsemble, _partition_estimators
 from ..base import ClassifierMixin, RegressorMixin
-from ..externals.joblib import Parallel, delayed
+from ..utils._joblib import Parallel, delayed
 from ..externals.six import with_metaclass
 from ..externals.six.moves import zip
 from ..metrics import r2_score, accuracy_score
 from ..tree import DecisionTreeClassifier, DecisionTreeRegressor
 from ..utils import check_random_state, check_X_y, check_array, column_or_1d
+from ..utils import indices_to_mask, check_consistent_length
+from ..utils.metaestimators import if_delegate_has_method
+from ..utils.multiclass import check_classification_targets
 from ..utils.random import sample_without_replacement
 from ..utils.validation import has_fit_parameter, check_is_fitted
-from ..utils.fixes import bincount
-from ..utils.metaestimators import if_delegate_has_method
-
-from .base import BaseEnsemble, _partition_estimators
 
 
 __all__ = ["BaggingClassifier",
@@ -32,52 +32,67 @@ __all__ = ["BaggingClassifier",
 MAX_INT = np.iinfo(np.int32).max
 
 
+def _generate_indices(random_state, bootstrap, n_population, n_samples):
+    """Draw randomly sampled indices."""
+    # Draw sample indices
+    if bootstrap:
+        indices = random_state.randint(0, n_population, n_samples)
+    else:
+        indices = sample_without_replacement(n_population, n_samples,
+                                             random_state=random_state)
+
+    return indices
+
+
+def _generate_bagging_indices(random_state, bootstrap_features,
+                              bootstrap_samples, n_features, n_samples,
+                              max_features, max_samples):
+    """Randomly draw feature and sample indices."""
+    # Get valid random state
+    random_state = check_random_state(random_state)
+
+    # Draw indices
+    feature_indices = _generate_indices(random_state, bootstrap_features,
+                                        n_features, max_features)
+    sample_indices = _generate_indices(random_state, bootstrap_samples,
+                                       n_samples, max_samples)
+
+    return feature_indices, sample_indices
+
+
 def _parallel_build_estimators(n_estimators, ensemble, X, y, sample_weight,
-                               seeds, verbose):
+                               seeds, total_n_estimators, verbose):
     """Private function used to build a batch of estimators within a job."""
     # Retrieve settings
     n_samples, n_features = X.shape
-    max_samples = ensemble.max_samples
-    max_features = ensemble.max_features
-
-    if (not isinstance(max_samples, (numbers.Integral, np.integer)) and
-            (0.0 < max_samples <= 1.0)):
-        max_samples = int(max_samples * n_samples)
-
-    if (not isinstance(max_features, (numbers.Integral, np.integer)) and
-            (0.0 < max_features <= 1.0)):
-        max_features = int(max_features * n_features)
-
+    max_features = ensemble._max_features
+    max_samples = ensemble._max_samples
     bootstrap = ensemble.bootstrap
     bootstrap_features = ensemble.bootstrap_features
     support_sample_weight = has_fit_parameter(ensemble.base_estimator_,
                                               "sample_weight")
+    if not support_sample_weight and sample_weight is not None:
+        raise ValueError("The base estimator doesn't support sample weight")
 
     # Build estimators
     estimators = []
-    estimators_samples = []
     estimators_features = []
 
     for i in range(n_estimators):
         if verbose > 1:
-            print("building estimator %d of %d" % (i + 1, n_estimators))
+            print("Building estimator %d of %d for this parallel run "
+                  "(total %d)..." % (i + 1, n_estimators, total_n_estimators))
 
-        random_state = check_random_state(seeds[i])
-        seed = check_random_state(random_state.randint(MAX_INT))
-        estimator = ensemble._make_estimator(append=False)
+        random_state = np.random.RandomState(seeds[i])
+        estimator = ensemble._make_estimator(append=False,
+                                             random_state=random_state)
 
-        try:  # Not all estimator accept a random_state
-            estimator.set_params(random_state=seed)
-        except ValueError:
-            pass
-
-        # Draw features
-        if bootstrap_features:
-            features = random_state.randint(0, n_features, max_features)
-        else:
-            features = sample_without_replacement(n_features,
-                                                  max_features,
-                                                  random_state=random_state)
+        # Draw random feature, sample indices
+        features, indices = _generate_bagging_indices(random_state,
+                                                      bootstrap_features,
+                                                      bootstrap, n_features,
+                                                      n_samples, max_features,
+                                                      max_samples)
 
         # Draw samples, using sample weights, and then fit
         if support_sample_weight:
@@ -87,40 +102,21 @@ def _parallel_build_estimators(n_estimators, ensemble, X, y, sample_weight,
                 curr_sample_weight = sample_weight.copy()
 
             if bootstrap:
-                indices = random_state.randint(0, n_samples, max_samples)
-                sample_counts = bincount(indices, minlength=n_samples)
+                sample_counts = np.bincount(indices, minlength=n_samples)
                 curr_sample_weight *= sample_counts
-
             else:
-                not_indices = sample_without_replacement(
-                    n_samples,
-                    n_samples - max_samples,
-                    random_state=random_state)
-
-                curr_sample_weight[not_indices] = 0
+                not_indices_mask = ~indices_to_mask(indices, n_samples)
+                curr_sample_weight[not_indices_mask] = 0
 
             estimator.fit(X[:, features], y, sample_weight=curr_sample_weight)
-            samples = curr_sample_weight > 0.
 
-        # Draw samples, using a mask, and then fit
         else:
-            if bootstrap:
-                indices = random_state.randint(0, n_samples, max_samples)
-            else:
-                indices = sample_without_replacement(n_samples,
-                                                     max_samples,
-                                                     random_state=random_state)
-
-            sample_counts = bincount(indices, minlength=n_samples)
-
             estimator.fit((X[indices])[:, features], y[indices])
-            samples = sample_counts > 0.
 
         estimators.append(estimator)
-        estimators_samples.append(samples)
         estimators_features.append(features)
 
-    return estimators, estimators_samples, estimators_features
+    return estimators, estimators_features
 
 
 def _parallel_predict_proba(estimators, estimators_features, X, n_classes):
@@ -204,7 +200,8 @@ class BaseBagging(with_metaclass(ABCMeta, BaseEnsemble)):
                  bootstrap=True,
                  bootstrap_features=False,
                  oob_score=False,
-                 n_jobs=1,
+                 warm_start=False,
+                 n_jobs=None,
                  random_state=None,
                  verbose=0):
         super(BaseBagging, self).__init__(
@@ -216,6 +213,7 @@ class BaseBagging(with_metaclass(ABCMeta, BaseEnsemble)):
         self.bootstrap = bootstrap
         self.bootstrap_features = bootstrap_features
         self.oob_score = oob_score
+        self.warm_start = warm_start
         self.n_jobs = n_jobs
         self.random_state = random_state
         self.verbose = verbose
@@ -242,47 +240,130 @@ class BaseBagging(with_metaclass(ABCMeta, BaseEnsemble)):
         Returns
         -------
         self : object
-            Returns self.
+        """
+        return self._fit(X, y, self.max_samples, sample_weight=sample_weight)
+
+    def _fit(self, X, y, max_samples=None, max_depth=None, sample_weight=None):
+        """Build a Bagging ensemble of estimators from the training
+           set (X, y).
+
+        Parameters
+        ----------
+        X : {array-like, sparse matrix} of shape = [n_samples, n_features]
+            The training input samples. Sparse matrices are accepted only if
+            they are supported by the base estimator.
+
+        y : array-like, shape = [n_samples]
+            The target values (class labels in classification, real numbers in
+            regression).
+
+        max_samples : int or float, optional (default=None)
+            Argument to use instead of self.max_samples.
+
+        max_depth : int, optional (default=None)
+            Override value used when constructing base estimator. Only
+            supported if the base estimator has a max_depth parameter.
+
+        sample_weight : array-like, shape = [n_samples] or None
+            Sample weights. If None, then samples are equally weighted.
+            Note that this is supported only if the base estimator supports
+            sample weighting.
+
+        Returns
+        -------
+        self : object
         """
         random_state = check_random_state(self.random_state)
 
-        # Convert data
-        X, y = check_X_y(X, y, ['csr', 'csc', 'coo'])
+        # Convert data (X is required to be 2d and indexable)
+        X, y = check_X_y(
+            X, y, ['csr', 'csc'], dtype=None, force_all_finite=False,
+            multi_output=True
+        )
+        if sample_weight is not None:
+            sample_weight = check_array(sample_weight, ensure_2d=False)
+            check_consistent_length(y, sample_weight)
 
         # Remap output
         n_samples, self.n_features_ = X.shape
+        self._n_samples = n_samples
         y = self._validate_y(y)
 
         # Check parameters
         self._validate_estimator()
 
-        if isinstance(self.max_samples, (numbers.Integral, np.integer)):
+        if max_depth is not None:
+            self.base_estimator_.max_depth = max_depth
+
+        # Validate max_samples
+        if max_samples is None:
             max_samples = self.max_samples
-        else:  # float
-            max_samples = int(self.max_samples * X.shape[0])
+        elif not isinstance(max_samples, (numbers.Integral, np.integer)):
+            max_samples = int(max_samples * X.shape[0])
 
         if not (0 < max_samples <= X.shape[0]):
             raise ValueError("max_samples must be in (0, n_samples]")
 
+        # Store validated integer row sampling value
+        self._max_samples = max_samples
+
+        # Validate max_features
         if isinstance(self.max_features, (numbers.Integral, np.integer)):
             max_features = self.max_features
-        else:  # float
-            max_features = int(self.max_features * self.n_features_)
+        elif isinstance(self.max_features, np.float):
+            max_features = self.max_features * self.n_features_
+        else:
+            raise ValueError("max_features must be int or float")
 
         if not (0 < max_features <= self.n_features_):
             raise ValueError("max_features must be in (0, n_features]")
 
+        max_features = max(1, int(max_features))
+
+        # Store validated integer feature sampling value
+        self._max_features = max_features
+
+        # Other checks
         if not self.bootstrap and self.oob_score:
             raise ValueError("Out of bag estimation only available"
                              " if bootstrap=True")
 
-        # Free allocated memory, if any
-        self.estimators_ = None
+        if self.warm_start and self.oob_score:
+            raise ValueError("Out of bag estimate only available"
+                             " if warm_start=False")
+
+        if hasattr(self, "oob_score_") and self.warm_start:
+            del self.oob_score_
+
+        if not self.warm_start or not hasattr(self, 'estimators_'):
+            # Free allocated memory, if any
+            self.estimators_ = []
+            self.estimators_features_ = []
+
+        n_more_estimators = self.n_estimators - len(self.estimators_)
+
+        if n_more_estimators < 0:
+            raise ValueError('n_estimators=%d must be larger or equal to '
+                             'len(estimators_)=%d when warm_start==True'
+                             % (self.n_estimators, len(self.estimators_)))
+
+        elif n_more_estimators == 0:
+            warn("Warm-start fitting without increasing n_estimators does not "
+                 "fit new trees.")
+            return self
 
         # Parallel loop
-        n_jobs, n_estimators, starts = _partition_estimators(self.n_estimators,
+        n_jobs, n_estimators, starts = _partition_estimators(n_more_estimators,
                                                              self.n_jobs)
-        seeds = random_state.randint(MAX_INT, size=self.n_estimators)
+        total_n_estimators = sum(n_estimators)
+
+        # Advance random state to state after training
+        # the first n_estimators
+        if self.warm_start and len(self.estimators_) > 0:
+            random_state.randint(MAX_INT, size=len(self.estimators_))
+
+        seeds = random_state.randint(MAX_INT, size=n_more_estimators)
+        self._seeds = seeds
 
         all_results = Parallel(n_jobs=n_jobs, verbose=self.verbose)(
             delayed(_parallel_build_estimators)(
@@ -292,16 +373,15 @@ class BaseBagging(with_metaclass(ABCMeta, BaseEnsemble)):
                 y,
                 sample_weight,
                 seeds[starts[i]:starts[i + 1]],
+                total_n_estimators,
                 verbose=self.verbose)
             for i in range(n_jobs))
 
         # Reduce
-        self.estimators_ = list(itertools.chain.from_iterable(
+        self.estimators_ += list(itertools.chain.from_iterable(
             t[0] for t in all_results))
-        self.estimators_samples_ = list(itertools.chain.from_iterable(
+        self.estimators_features_ += list(itertools.chain.from_iterable(
             t[1] for t in all_results))
-        self.estimators_features_ = list(itertools.chain.from_iterable(
-            t[2] for t in all_results))
 
         if self.oob_score:
             self._set_oob_score(X, y)
@@ -313,8 +393,38 @@ class BaseBagging(with_metaclass(ABCMeta, BaseEnsemble)):
         """Calculate out of bag predictions and score."""
 
     def _validate_y(self, y):
-        # Default implementation
-        return column_or_1d(y, warn=True)
+        if len(y.shape) == 1 or y.shape[1] == 1:
+            return column_or_1d(y, warn=True)
+        else:
+            return y
+
+    def _get_estimators_indices(self):
+        # Get drawn indices along both sample and feature axes
+        for seed in self._seeds:
+            # Operations accessing random_state must be performed identically
+            # to those in `_parallel_build_estimators()`
+            random_state = np.random.RandomState(seed)
+            feature_indices, sample_indices = _generate_bagging_indices(
+                random_state, self.bootstrap_features, self.bootstrap,
+                self.n_features_, self._n_samples, self._max_features,
+                self._max_samples)
+
+            yield feature_indices, sample_indices
+
+    @property
+    def estimators_samples_(self):
+        """The subset of drawn samples for each base estimator.
+
+        Returns a dynamically generated list of indices identifying
+        the samples used for fitting each member of the ensemble, i.e.,
+        the in-bag samples.
+
+        Note: the list is re-created at each call to the property in order
+        to reduce the object memory footprint by not storing the sampling
+        data. Thus fetching the property may be slower than expected.
+        """
+        return [sample_indices
+                for _, sample_indices in self._get_estimators_indices()]
 
 
 class BaggingClassifier(BaseBagging, ClassifierMixin):
@@ -337,6 +447,8 @@ class BaggingClassifier(BaseBagging, ClassifierMixin):
     on subsets of both samples and features, then the method is known as
     Random Patches [4]_.
 
+    Read more in the :ref:`User Guide <bagging>`.
+
     Parameters
     ----------
     base_estimator : object or None, optional (default=None)
@@ -348,27 +460,40 @@ class BaggingClassifier(BaseBagging, ClassifierMixin):
 
     max_samples : int or float, optional (default=1.0)
         The number of samples to draw from X to train each base estimator.
-            - If int, then draw `max_samples` samples.
-            - If float, then draw `max_samples * X.shape[0]` samples.
+
+        - If int, then draw `max_samples` samples.
+        - If float, then draw `max_samples * X.shape[0]` samples.
 
     max_features : int or float, optional (default=1.0)
         The number of features to draw from X to train each base estimator.
-            - If int, then draw `max_features` features.
-            - If float, then draw `max_features * X.shape[1]` features.
+
+        - If int, then draw `max_features` features.
+        - If float, then draw `max_features * X.shape[1]` features.
 
     bootstrap : boolean, optional (default=True)
-        Whether samples are drawn with replacement.
+        Whether samples are drawn with replacement. If False, sampling
+        without replacement is performed.
 
     bootstrap_features : boolean, optional (default=False)
         Whether features are drawn with replacement.
 
-    oob_score : bool
+    oob_score : bool, optional (default=False)
         Whether to use out-of-bag samples to estimate
         the generalization error.
 
-    n_jobs : int, optional (default=1)
+    warm_start : bool, optional (default=False)
+        When set to True, reuse the solution of the previous call to fit
+        and add more estimators to the ensemble, otherwise, just fit
+        a whole new ensemble. See :term:`the Glossary <warm_start>`.
+
+        .. versionadded:: 0.17
+           *warm_start* constructor parameter.
+
+    n_jobs : int or None, optional (default=None)
         The number of jobs to run in parallel for both `fit` and `predict`.
-        If -1, then the number of jobs is set to the number of cores.
+        ``None`` means 1 unless in a :obj:`joblib.parallel_backend` context.
+        ``-1`` means using all processors. See :term:`Glossary <n_jobs>`
+        for more details.
 
     random_state : int, RandomState instance or None, optional (default=None)
         If int, random_state is the seed used by the random number generator;
@@ -377,11 +502,11 @@ class BaggingClassifier(BaseBagging, ClassifierMixin):
         by `np.random`.
 
     verbose : int, optional (default=0)
-        Controls the verbosity of the building process.
+        Controls the verbosity when fitting and predicting.
 
     Attributes
     ----------
-    base_estimator_ : list of estimators
+    base_estimator_ : estimator
         The base estimator from which the ensemble is grown.
 
     estimators_ : list of estimators
@@ -389,7 +514,7 @@ class BaggingClassifier(BaseBagging, ClassifierMixin):
 
     estimators_samples_ : list of arrays
         The subset of drawn samples (i.e., the in-bag samples) for each base
-        estimator.
+        estimator. Each subset is defined by an array of the indices selected.
 
     estimators_features_ : list of arrays
         The subset of drawn features for each base estimator.
@@ -433,7 +558,8 @@ class BaggingClassifier(BaseBagging, ClassifierMixin):
                  bootstrap=True,
                  bootstrap_features=False,
                  oob_score=False,
-                 n_jobs=1,
+                 warm_start=False,
+                 n_jobs=None,
                  random_state=None,
                  verbose=0):
 
@@ -445,6 +571,7 @@ class BaggingClassifier(BaseBagging, ClassifierMixin):
             bootstrap=bootstrap,
             bootstrap_features=bootstrap_features,
             oob_score=oob_score,
+            warm_start=warm_start,
             n_jobs=n_jobs,
             random_state=random_state,
             verbose=verbose)
@@ -455,17 +582,16 @@ class BaggingClassifier(BaseBagging, ClassifierMixin):
             default=DecisionTreeClassifier())
 
     def _set_oob_score(self, X, y):
-        n_classes_ = self.n_classes_
-        classes_ = self.classes_
         n_samples = y.shape[0]
+        n_classes_ = self.n_classes_
 
         predictions = np.zeros((n_samples, n_classes_))
 
         for estimator, samples, features in zip(self.estimators_,
                                                 self.estimators_samples_,
                                                 self.estimators_features_):
-            mask = np.ones(n_samples, dtype=np.bool)
-            mask[samples] = False
+            # Create mask for OOB samples
+            mask = ~indices_to_mask(samples, n_samples)
 
             if hasattr(estimator, "predict_proba"):
                 predictions[mask, :] += estimator.predict_proba(
@@ -487,14 +613,14 @@ class BaggingClassifier(BaseBagging, ClassifierMixin):
 
         oob_decision_function = (predictions /
                                  predictions.sum(axis=1)[:, np.newaxis])
-        oob_score = accuracy_score(y, classes_.take(np.argmax(predictions,
-                                                              axis=1)))
+        oob_score = accuracy_score(y, np.argmax(predictions, axis=1))
 
         self.oob_decision_function_ = oob_decision_function
         self.oob_score_ = oob_score
 
     def _validate_y(self, y):
         y = column_or_1d(y, warn=True)
+        check_classification_targets(y)
         self.classes_, y = np.unique(y, return_inverse=True)
         self.n_classes_ = len(self.classes_)
 
@@ -529,7 +655,7 @@ class BaggingClassifier(BaseBagging, ClassifierMixin):
         the mean predicted class probabilities of the base estimators in the
         ensemble. If base estimators do not implement a ``predict_proba``
         method, then it resorts to voting and the predicted class probabilities
-        of a an input sample represents the proportion of estimators predicting
+        of an input sample represents the proportion of estimators predicting
         each class.
 
         Parameters
@@ -546,7 +672,10 @@ class BaggingClassifier(BaseBagging, ClassifierMixin):
         """
         check_is_fitted(self, "classes_")
         # Check data
-        X = check_array(X, accept_sparse=['csr', 'csc', 'coo'])
+        X = check_array(
+            X, accept_sparse=['csr', 'csc'], dtype=None,
+            force_all_finite=False
+        )
 
         if self.n_features_ != X.shape[1]:
             raise ValueError("Number of features of the model must "
@@ -593,7 +722,10 @@ class BaggingClassifier(BaseBagging, ClassifierMixin):
         check_is_fitted(self, "classes_")
         if hasattr(self.base_estimator_, "predict_log_proba"):
             # Check data
-            X = check_array(X)
+            X = check_array(
+                X, accept_sparse=['csr', 'csc'], dtype=None,
+                force_all_finite=False
+            )
 
             if self.n_features_ != X.shape[1]:
                 raise ValueError("Number of features of the model must "
@@ -648,12 +780,15 @@ class BaggingClassifier(BaseBagging, ClassifierMixin):
         check_is_fitted(self, "classes_")
 
         # Check data
-        X = check_array(X)
+        X = check_array(
+            X, accept_sparse=['csr', 'csc'], dtype=None,
+            force_all_finite=False
+        )
 
         if self.n_features_ != X.shape[1]:
             raise ValueError("Number of features of the model must "
-                             "match the input. Model n_features is {1} and "
-                             "input n_features is {2} "
+                             "match the input. Model n_features is {0} and "
+                             "input n_features is {1} "
                              "".format(self.n_features_, X.shape[1]))
 
         # Parallel loop
@@ -693,6 +828,8 @@ class BaggingRegressor(BaseBagging, RegressorMixin):
     on subsets of both samples and features, then the method is known as
     Random Patches [4]_.
 
+    Read more in the :ref:`User Guide <bagging>`.
+
     Parameters
     ----------
     base_estimator : object or None, optional (default=None)
@@ -704,16 +841,19 @@ class BaggingRegressor(BaseBagging, RegressorMixin):
 
     max_samples : int or float, optional (default=1.0)
         The number of samples to draw from X to train each base estimator.
-            - If int, then draw `max_samples` samples.
-            - If float, then draw `max_samples * X.shape[0]` samples.
+
+        - If int, then draw `max_samples` samples.
+        - If float, then draw `max_samples * X.shape[0]` samples.
 
     max_features : int or float, optional (default=1.0)
         The number of features to draw from X to train each base estimator.
-            - If int, then draw `max_features` features.
-            - If float, then draw `max_features * X.shape[1]` features.
+
+        - If int, then draw `max_features` features.
+        - If float, then draw `max_features * X.shape[1]` features.
 
     bootstrap : boolean, optional (default=True)
-        Whether samples are drawn with replacement.
+        Whether samples are drawn with replacement. If False, sampling
+        without replacement is performed.
 
     bootstrap_features : boolean, optional (default=False)
         Whether features are drawn with replacement.
@@ -722,9 +862,16 @@ class BaggingRegressor(BaseBagging, RegressorMixin):
         Whether to use out-of-bag samples to estimate
         the generalization error.
 
-    n_jobs : int, optional (default=1)
+    warm_start : bool, optional (default=False)
+        When set to True, reuse the solution of the previous call to fit
+        and add more estimators to the ensemble, otherwise, just fit
+        a whole new ensemble. See :term:`the Glossary <warm_start>`.
+
+    n_jobs : int or None, optional (default=None)
         The number of jobs to run in parallel for both `fit` and `predict`.
-        If -1, then the number of jobs is set to the number of cores.
+        ``None`` means 1 unless in a :obj:`joblib.parallel_backend` context.
+        ``-1`` means using all processors. See :term:`Glossary <n_jobs>`
+        for more details.
 
     random_state : int, RandomState instance or None, optional (default=None)
         If int, random_state is the seed used by the random number generator;
@@ -733,7 +880,7 @@ class BaggingRegressor(BaseBagging, RegressorMixin):
         by `np.random`.
 
     verbose : int, optional (default=0)
-        Controls the verbosity of the building process.
+        Controls the verbosity when fitting and predicting.
 
     Attributes
     ----------
@@ -742,7 +889,7 @@ class BaggingRegressor(BaseBagging, RegressorMixin):
 
     estimators_samples_ : list of arrays
         The subset of drawn samples (i.e., the in-bag samples) for each base
-        estimator.
+        estimator. Each subset is defined by an array of the indices selected.
 
     estimators_features_ : list of arrays
         The subset of drawn features for each base estimator.
@@ -781,7 +928,8 @@ class BaggingRegressor(BaseBagging, RegressorMixin):
                  bootstrap=True,
                  bootstrap_features=False,
                  oob_score=False,
-                 n_jobs=1,
+                 warm_start=False,
+                 n_jobs=None,
                  random_state=None,
                  verbose=0):
         super(BaggingRegressor, self).__init__(
@@ -792,6 +940,7 @@ class BaggingRegressor(BaseBagging, RegressorMixin):
             bootstrap=bootstrap,
             bootstrap_features=bootstrap_features,
             oob_score=oob_score,
+            warm_start=warm_start,
             n_jobs=n_jobs,
             random_state=random_state,
             verbose=verbose)
@@ -815,7 +964,10 @@ class BaggingRegressor(BaseBagging, RegressorMixin):
         """
         check_is_fitted(self, "estimators_features_")
         # Check data
-        X = check_array(X, accept_sparse=['csr', 'csc', 'coo'])
+        X = check_array(
+            X, accept_sparse=['csr', 'csc'], dtype=None,
+            force_all_finite=False
+        )
 
         # Parallel loop
         n_jobs, n_estimators, starts = _partition_estimators(self.n_estimators,
@@ -847,8 +999,8 @@ class BaggingRegressor(BaseBagging, RegressorMixin):
         for estimator, samples, features in zip(self.estimators_,
                                                 self.estimators_samples_,
                                                 self.estimators_features_):
-            mask = np.ones(n_samples, dtype=np.bool)
-            mask[samples] = False
+            # Create mask for OOB samples
+            mask = ~indices_to_mask(samples, n_samples)
 
             predictions[mask] += estimator.predict((X[mask, :])[:, features])
             n_predictions[mask] += 1
